//***************************************************************
// TITLE    :    APE6-EVM-BOARD TEST Program
// FILE NAME:    printer.c
//
// RENESAS ELECTRONICS CONFIDENTIAL AND PROPRIETARY
// Copyright (C) 2012 Renesas Mobile Corp.
// All rights reserved.
//
// FUNC:         process monitor output
// NOTE:         -
// LAST UPDATE:  2012/10/9
//**************************************************************

//*************************************************************
// INCLUDE FILE                                                
//*************************************************************
#include <stdio.h>
#include <string.h>
#include <stdarg.h>


static void ETI_i2x(uint32_t,char*);
static void ETI_i2u(uint32_t,char*);
static void ETI_i2a(int,char*);
static void ETI_i2b(uint32_t,char*);
static void ETI_Reverse(char*);


void ETG_MonPrintf(const char *format,...);
void ETG_COM_Debuglog(const char *format,...);
void ETG_DebugPrintf(const char *format ,... );


//***********************************************************************************************
// FUNCTION : ETG_MonPrintf(const char *format ,... )                                                
// ARGUMENT : %02/04/08xXdDuUsScC
//						x or X : hex
//						d or D : decimal
//						u or U : unsigned int
//						s or S : string
//						c or C : 1 character
// RETURN   : none
// OUTLINE  : similar to printf()
//            No debug-log output
// history  : 2012/10/2 Update                                                              
//***********************************************************************************************
void ETG_MonPrintf(const char *format ,... )
{
	va_list ap;
	uint8_t*	puint8Format = (uint8_t*)format;
	uint8_t		uint8Sdigit[34];
	uint8_t*	puint8Str;

	va_start( ap, format );

	while (*puint8Format != '\0')
	{
		if (*puint8Format == '%')
		{
			puint8Format++;
			switch ( *puint8Format )
			{
				case '0':
				puint8Format++;
				switch ( *puint8Format )
				{
					case '1':
						puint8Format++;
						switch (*puint8Format)
						{
							case 'b':case 'B':		// %b01
								ETI_i2b( va_arg(ap,uint32_t), (char*)uint8Sdigit );
								ETG_MonWrite( (uint8_t*)uint8Sdigit+31, 1 );
								break;
							case '2':
							puint8Format++;
							switch (*puint8Format)
							{
								case 'b':case 'B':		// %b012
									ETI_i2b( va_arg(ap,uint32_t), (char*)uint8Sdigit );
									ETG_MonWrite( (uint8_t*)uint8Sdigit+20, 12 );
									break;
								default:
									ETG_MonWrite( (uint8_t*)puint8Format, 1 );
							}
							case '6':
							puint8Format++;
							switch (*puint8Format)
							{
								case 'b':case 'B':		// %b016
									ETI_i2b( va_arg(ap,uint32_t), (char*)uint8Sdigit );
									ETG_MonWrite( (uint8_t*)uint8Sdigit+16, 16 );
									break;
								default:
									ETG_MonWrite( (uint8_t*)puint8Format, 1 );
							}
							break;
						}
						break;
					case '2':
						puint8Format++;
						switch (*puint8Format)
						{
							case '0':
							puint8Format++;
							switch (*puint8Format)
							{
								case 'b':case 'B':		// %b020
									ETI_i2b( va_arg(ap,uint32_t), (char*)uint8Sdigit );
									ETG_MonWrite( (uint8_t*)uint8Sdigit+12, 20 );
									break;
								default:
									ETG_MonWrite( (uint8_t*)puint8Format, 1 );
							}
							case '4':
							puint8Format++;
							switch (*puint8Format)
							{
								case 'b':case 'B':		// %b024
									ETI_i2b( va_arg(ap,uint32_t), (char*)uint8Sdigit );
									ETG_MonWrite( (uint8_t*)uint8Sdigit+8, 24 );
									break;
								default:
									ETG_MonWrite( (uint8_t*)puint8Format, 1 );
							}
							case '8':
							puint8Format++;
							switch (*puint8Format)
							{
								case 'b':case 'B':		// %b028
									ETI_i2b( va_arg(ap,uint32_t), (char*)uint8Sdigit );
									ETG_MonWrite( (uint8_t*)uint8Sdigit+4, 28 );
									break;
								default:
									ETG_MonWrite( (uint8_t*)puint8Format, 1 );
							}
							break;
							case 'x':case 'X':
								ETI_i2x( va_arg(ap,uint32_t), (char*)uint8Sdigit );
								ETG_MonWrite( (uint8_t*)uint8Sdigit+6, 2 );
								break;
							case 'u':case 'U':
								ETI_i2u( va_arg(ap,uint32_t), (char*)uint8Sdigit );
								ETG_MonWrite( (uint8_t*)uint8Sdigit+6, 2 );
								break;
							case 'b':case 'B':		// %b02
								ETI_i2b( va_arg(ap,uint32_t), (char*)uint8Sdigit );
								ETG_MonWrite( (uint8_t*)uint8Sdigit+30, 2 );
								break;
							default:
								ETG_MonWrite( (uint8_t*)puint8Format, 1 );
						}
						break;
					case '3':
						puint8Format++;
						switch (*puint8Format)
						{
							case '2':
							puint8Format++;
							switch (*puint8Format)
							{
								case 'b':case 'B':		// %b032
									ETI_i2b( va_arg(ap,uint32_t), (char*)uint8Sdigit );
									ETG_MonWrite( (uint8_t*)uint8Sdigit, 32 );
									break;
								default:
									ETG_MonWrite( (uint8_t*)puint8Format, 1 );
							}
							break;
						}
						break;
					case '4':
						puint8Format++;
						switch ( *puint8Format )
						{
							case 'x':case 'X':
								ETI_i2x( va_arg( ap,uint32_t ), (char*)uint8Sdigit );
								ETG_MonWrite( (uint8_t*)uint8Sdigit+4, 4 );
								break;
							case 'u':case 'U':
								ETI_i2u( va_arg(ap,uint32_t), (char*)uint8Sdigit );
								ETG_MonWrite( (uint8_t*)uint8Sdigit+4, 4 );
								break;
							case 'd':case 'D':
								ETI_i2a( va_arg(ap,int), (char*)uint8Sdigit );
								ETG_MonWrite( (uint8_t*)uint8Sdigit+4, 4 );
								break;
							case 'b':case 'B':
								ETI_i2b( va_arg(ap,uint32_t), (char*)uint8Sdigit );
								ETG_MonWrite( (uint8_t*)uint8Sdigit+28, 4 );
								break;
							default:
								ETG_MonWrite( (uint8_t*)puint8Format, 1 );
						}
						break;
					case '8':
						puint8Format++;
						switch ( *puint8Format )
						{
							case 'x':case 'X':
								ETI_i2x( va_arg(ap,uint32_t), (char*)uint8Sdigit );
								ETG_MonWrite( (uint8_t*)uint8Sdigit, 8 );
								break;
							case 'u':case 'U':
								ETI_i2u( va_arg(ap,uint32_t), (char*)uint8Sdigit );
								ETG_MonWrite( (uint8_t*)uint8Sdigit, 8 );
								break;
							case 'b':case 'B':
								ETI_i2b( va_arg(ap,uint32_t), (char*)uint8Sdigit );
								ETG_MonWrite( (uint8_t*)uint8Sdigit+24, 8 );
								break;
							default:
								ETG_MonWrite( (uint8_t*)puint8Format, 1 );
						}
						break;
					default:
						ETG_MonWrite( (uint8_t*)puint8Format, 1 );
					}
					break;
				case 'x':case 'X':
					ETI_i2x( va_arg(ap,uint32_t), (char*)uint8Sdigit );
					puint8Str = uint8Sdigit;
					while( (*puint8Str) == '0' )
					{
						puint8Str++;
					}
					if( *puint8Str == '\0' )
					{
						puint8Str--;
					}
					ETG_MonWrite( (uint8_t*)puint8Str, l_strlen( (char*)puint8Str) );
					break;
				case 'u':case 'U':
					ETI_i2u( va_arg(ap,uint32_t), (char*)uint8Sdigit );
					puint8Str = uint8Sdigit;
					while ( (*puint8Str) == '0' )
					{
						puint8Str++;
					}
					if( *puint8Str == '\0' )
					{
						puint8Str--;
					}
					ETG_MonWrite( (uint8_t*)puint8Str, l_strlen( (char*)puint8Str) );
					break;
				case 'd':case 'D':
					ETI_i2a( va_arg(ap,int), (char*)uint8Sdigit );
					ETG_MonWrite( (uint8_t*)uint8Sdigit, l_strlen( (char*)uint8Sdigit) );
					break;
				case 's':case 'S':
					puint8Str = (uint8_t*)va_arg( ap,char* );
					ETG_MonWrite( (uint8_t*)puint8Str, l_strlen( (char*)puint8Str ) );
					break;
				case 'c':case 'C':
					uint8Sdigit[0] = va_arg( ap, int );
					ETG_MonWrite( (uint8_t*)&uint8Sdigit[0], 1 );
					break;
				case 'b':case 'B':
					ETI_i2b( va_arg(ap,uint32_t), (char*)uint8Sdigit );
					puint8Str = uint8Sdigit;
					while( (*puint8Str) == '0' )
					{
						puint8Str++;
					}
					if( *puint8Str == '\0' )
					{
						puint8Str--;
					}
					ETG_MonWrite( (uint8_t*)puint8Str, l_strlen( (char*)puint8Str) );
					break;
				default:
					ETG_MonWrite( (uint8_t*)puint8Format, 1 );
			}
			puint8Format++;
		}
		else{
			ETG_MonWrite( (uint8_t*)puint8Format, 1 );
			puint8Format++;
		}
	}
	va_end(ap);

	return;
}

//***********************************************************************************************
// FUNCTION : ETI_i2x(uint32_t n ,char s[] )                                               
// ARGUMENT : uint32_t n ,char s[]                                                          
// RETURN   : static void                                                                        
// OUTLINE  : decimal to hex
//            No debug-log output
// history  : 2012/10/2 Update                                                          
//***********************************************************************************************
static void ETI_i2x( uint32_t n ,char s[] )
{
	int	i = 0;
	int	a;
	
	do {
		if ((a = (n % 16)) < 10) {
			s[i++] = a + '0';
		}
		else {
			s[i++] = a + '7';
		}
		n /= 16;
	}
	while(i < 8);
	s[i] = '\0';
	ETI_Reverse(s);
}

//***********************************************************************************************
// FUNCTION : ETI_i2u(uint32_t n ,char s[] )                                               
// ARGUMENT : uint32_t n ,char s[]                                                          
// RETURN   : static void                                                                        
// OUTLINE  : ULONG->string              
//            No debug-log output                                                
// history  : 2012/10/2 Update                                                   
//***********************************************************************************************
static void ETI_i2u( uint32_t n , char s[] )
{
	int	i = 0;

	do {
		s[i++] = (n % 10) + '0';
		n /= 10;
	}
	while(i < 8);
	s[i] = '\0';
	ETI_Reverse(s);
}

//***********************************************************************************************
// FUNCTION : ETI_i2a(int n ,char s[] )                                                         
// ARGUMENT : int n ,char s[]                                                                    
// RETURN   : static void                                                                        
// OUTLINE  : int->string     
//            No debug-log output                                                         
// history  : 2012/10/2 Update  
//***********************************************************************************************
static void ETI_i2a( int n ,char s[] )
{
	int	i ,sign;

	if ((sign = n) < 0) {
		n = -n;
	}
	i = 0;
	do {
		s[i++] = (n % 10) + '0';
	}
	while((n /= 10) > 0);
	if (sign < 0) {
		s[i++] = '-';
	}
	s[i] = '\0';
	ETI_Reverse(s);
}

//***********************************************************************************************
// FUNCTION : ETI_i2b( uint32_t n ,char s[] )                                               
// ARGUMENT : uint32_t n ,char s[]                                                          
// RETURN   : static void                                                                        
// OUTLINE  : binary -> string              
//            No debug-log output                                                
// history  : 2012/12/06 Update                                                   
//***********************************************************************************************
static void ETI_i2b( uint32_t n , char s[] )
{
	uint32_t i;

	for( i=0; i<32; i++ ){
		if( (n>>i)&1 ) s[i] = '1';
		else		   s[i] = '0';
	}
	s[i] = '\0';
	ETI_Reverse(s);
}

//***********************************************************************************************
// FUNCTION : ETI_Reverse(char s[])                                                              
// ARGUMENT : char s[]                                                                           
// RETURN   : static void                                                                        
// OUTLINE  : reverse string
//            No debug-log output
// history  : 2012/10/2 Update  
//***********************************************************************************************
static void ETI_Reverse( char s[] )
{
	int	c,i,j;

	for ( i = 0, j = l_strlen(s)-1; i < j; i++, j--) {
		c = s[i];
		s[i] = s[j];
		s[j] = c;
	}
}
