#include "stm32f10x_conf.h"

void USART_SendString( const char *str );


void RegWrite32(unsigned int addr, unsigned int X){
	*(volatile unsigned long *)(addr)  = (unsigned long)(X);
}
void RegWrite16(unsigned int addr, unsigned int X){
	*(volatile unsigned short *)(addr) = (unsigned short)(X);
}
void RegWrite8 (unsigned int addr, unsigned int X){
	*(volatile unsigned char *)(addr)  = (unsigned char)(X);
}


int RegRead32 (unsigned int addr){ return *(volatile unsigned long *)(addr);}
int RegRead16 (unsigned int addr){ return *(volatile unsigned short *)(addr);}
int RegRead8  (unsigned int addr){ return *(volatile unsigned char *)(addr);}




/*
 *============================================================================
 *  [֐]: putc
 *  [@\]  : UART01oCgo
 *  []  : 
 *  [߂l]: 
 *  [l]  : 
 *============================================================================
*/
void putc( unsigned char c )
{
	if ( c == '\n') {
		while(USART_GetFlagStatus(USART1, USART_FLAG_TXE) == RESET) {
		}
		USART_SendData(USART1,  '\r');
	}

	while(USART_GetFlagStatus(USART1, USART_FLAG_TXE) == RESET) {
	}
	/* Send character */
	USART_SendData(USART1,  c);
}


//----------------------------------------------------------
//  [֐]: putchar
//  [@\]  : UART01oCgo
//----------------------------------------------------------
void putchar( char  c )
{
	putc( c );
}

//----------------------------------------------------------
//  [֐]: getchar
//  [@\]  : UART01
//----------------------------------------------------------
char getchar( void )
{
	char data;

	while( USART_GetITStatus(USART1, USART_IT_RXNE) == 0 ) {};
    data = (char)USART_ReceiveData(USART1);
	return( data );
}

//----------------------------------------------------------
//  [֐]: dbg_print
//  [@\]  : Ȉprintf
//----------------------------------------------------------
void dbg_print(const char* s)
{
	for ( ; *s != '\0'; s++) {
		if (*s == '\n') {
			putc('\r');
			putc('\n');
		} else {
			putc(*s);
		}
	}
}

/*
 *============================================================================
 *  [֐]: hex2char
 *  [@\]  : 
 *  []  : 
 *  [߂l]: 
 *  [l]  : 
 *============================================================================
*/
static void hex2char( unsigned long val )
{
	char			buf[10];
	char			*tmp_ptr = buf;
	int				i=1;
	unsigned int	tmp, uint16=val;
	
	while( 1 ) {
		uint16 >>= 4;
		if( uint16 == 0 ) {
			break;
		}
		i++;
	}
	*(tmp_ptr + i) = '\0';
	
	tmp_ptr += (i - 1);
	uint16 = val;
	while( i ) {
		tmp = uint16 & 0x0000000F;
		if( tmp < 10 ) {
			*tmp_ptr-- = (char)('0' + tmp);
		} else {
			*tmp_ptr-- = (char)('a' + tmp - 10);
		}
		uint16 >>= 4;
		i--;
	}
	dbg_print(buf);
}



//----------------------------------------------------------
//  [֐]: printf_sub
//----------------------------------------------------------
void printf_sub(char s, long num, signed char count, char pad)
{
	long			swNum	= 0;
	unsigned long	uwNum	= 0;
	unsigned long	uwDig	= 0;
	char			buf[11]	= "";
	char			*ptr	= 0;
	char			flag	= 0;

	switch (s) {
		case 'd':
			swNum	= num;
			ptr		= buf;
			if (swNum < 0) {
				swNum = -swNum;
				flag = 1;
			}
			do {
				*ptr++ = '0' + swNum % 10;
				count--;
			} while (swNum /= 10);
			while (count-- > 0) {
				*ptr++ = pad;
			}
			if (flag) {
				putchar('-');
			}
			do {
				putchar(*--ptr);
			} while (ptr != buf);
			break;
		case 'x':
			uwNum	= num;
			ptr		= buf;
			do {
				*ptr++ = ((uwDig = (uwNum & 0xF)) > 9) ? ('a' + uwDig - 10) : ('0' + uwDig);
				count--;
			} while (uwNum >>= 4);
			while (count-- > 0) {
				*ptr++ = pad;
			}
			do {
				putchar(*--ptr);
			} while (ptr != buf);
			break;
		case 'X':
			uwNum	= num;
			ptr		= buf;
			do {
				*ptr++ = ((uwDig = (uwNum & 0xF)) > 9) ? ('A' + uwDig - 10) : ('0' + uwDig);
				count--;
			} while (uwNum >>= 4);
			while (count-- > 0) {
				*ptr++ = pad;
			}
			do {
				putchar(*--ptr);
			} while (ptr != buf);
	}
}

//----------------------------------------------------------
//  [֐]: lprintf
//----------------------------------------------------------
void lprintf(const char *format, ...)
{
	int *dataptr = (int *)&format;
	char c;

	dataptr++;
	while ((c = *format++) != 0) {
		if (c != '%') {
			putchar(c);
		}
		else {
			switch (c = *format++) {
				case 'd': {
					int num = *dataptr++;
					char buf[10], *ptr = buf;
					if (num < 0) {
						num = -num;
						putchar('-');
					}
					do {
						*ptr++ = '0' + num % 10;
					} while (num /= 10);
					do {
						putchar(*--ptr);
					} while (ptr != buf);
					break;
				}
				case 'u': {
					unsigned int num = *dataptr++;
					char buf[10], *ptr = buf;
					do {
						*ptr++ = '0' + num % 10;
					} while (num /= 10);
					do {
						putchar(*--ptr);
					} while (ptr != buf);
					break;
				}
				case 'x': {
					unsigned int num = *dataptr++, dig;
					char buf[8], *ptr = buf;
					do {
						*ptr++ = ((dig = (num & 0xF)) > 9) ? ('a' + dig - 10) : ('0' + dig);
					} while (num >>= 4);
					do {
						putchar(*--ptr);
					} while (ptr != buf);
					break;
				}
				case 'X': {
					unsigned int num = *dataptr++, dig;
					char buf[8], *ptr = buf;
					do {
						*ptr++ = ((dig = (num & 0xF)) > 9) ? ('A' + dig - 10) : ('0' + dig);
					} while (num >>= 4);
					do {
						putchar(*--ptr);
					} while (ptr != buf);
					break;
				}
				case 'c': {
					putchar((*dataptr++) & 0xFF);
					break;
				}
				case 's': {
					char *ptr = (char *)*dataptr++;
					while ((c = *ptr++) != 0) {
						putchar(c);
					}
					break;
				}
				case '0': {
					switch (c = *format++) {
						case '1': {
							char s = *format++;
							int num = *dataptr++;
							if (s == '0') {
								s = *format++;
								printf_sub(s, num, 10, '0');
							}
							else {
								printf_sub(s, num, 1, '0');
							}
							break;
						}
						case '2':
						case '3':
						case '4':
						case '5':
						case '6':
						case '7':
						case '8':
						case '9': {
							char s = *format++;
							int num = *dataptr++;
							printf_sub(s, num, c - '0', '0');
							break;
						}
					}
					break;
				}
				case '1': {
					char s = *format++;
					int num = *dataptr++;
					if (s == '0') {
						s = *format++;
						printf_sub(s, num, 10, ' ');
					}
					else {
						printf_sub(s, num, 1, ' ');
					}
					break;
				}
				case '2':
				case '3':
				case '4':
				case '5':
				case '6':
				case '7':
				case '8':
				case '9': {
					char s = *format++;
					int num = *dataptr++;
					printf_sub(s, num, c - '0', ' ');
					break;
				}
				case '%': {
					putchar('%' & 0xFF);
					break;
				}
			}
		}
	}
}


//----------------------------------------------------------
//  [֐]: dscanf
//----------------------------------------------------------
int dscanf(const char *format, unsigned int *uipt)
{

	char c;
	char buf[10];
	int	pt,i,mode;
	unsigned int tempui;
	
	if((c = *format++) != '%' )	return(0);
	switch(c = *format++){
		case 'd':	mode = 'd'; break;
		case 'x':	mode = 'x'; break;
		default	: return(0);
	}
	
	pt=0;
	while ( (c = getchar())!= '\r' ){
		if(c==0x1b){	// ESC
			return(0);
		}else if( (mode == 'd') &&  (c>='0' && c<='9') ){
			if(pt<10){
				buf[pt++] = c;
				putchar(c);
			}
		}else if((mode=='x') && ((c>='0' && c<='9') || (c>='A' && c<='F') ||(c>='a' && c<='f')) ){
			if(pt<8){
				buf[pt++] = c;
				putchar(c);
			}
		}else if(c==0x8||c==0x7f){	// BS or DEL
			if(pt>0){
				putchar(0x08);	// BS
				putchar(0x20);  // SPACE
				putchar(0x08);  // BS
				pt--;
			}
		}
	}
	
	if(pt==0) return(0);
	
	switch(mode){
		case 'd':{
			for(tempui=0,i=0;i<pt;i++){
				tempui = tempui*10 + buf[i] - '0';
				if(tempui>0x80000000) return(0);
			}
			break;
		}
		case 'x':{
			for(tempui=0,i=0;i<pt;i++){
				unsigned int ttt;
				if((buf[i]>='0') && (buf[i]<='9')) 	ttt = buf[i] - '0';
				else if ((buf[i]>='A') && (buf[i]<='F')) ttt = buf[i] - 'A'+ 10;
				else ttt = buf[i] - 'a'+ 10;
				
				tempui = (tempui<<4)+ttt;
			}
			break;
		}
		default:
				return(0);
	}
	
	*uipt = tempui;
	
	return(1);
}

//----------------------------------------------------------
//  [֐]: hex_dscanf
//----------------------------------------------------------
int hex_dscanf(void)
{

	char c;
	char buf[10];
	int	pt,i;
	unsigned int tempui;

	pt=0;
	while ( (c = getchar())!= '\r' ){
		if((c>='0' && c<='9') || (c>='A' && c<='F') ||(c>='a' && c<='f')){
			if(pt<8){
				buf[pt++] = c;
				putchar(c);
			}
		}else if(c==0x8||c==0x7f){
			if(pt>0){
				putchar(0x08);
				putchar(0x20);
				putchar(0x08);
				pt--;
			}
		}
	}
	if(pt==0) return(0);

	for(tempui=0,i=0;i<pt;i++){
		int ttt;
		if((buf[i]>='0') && (buf[i]<='9')) ttt = buf[i] - '0';
		else if ((buf[i]>='A') && (buf[i]<='F')) ttt = buf[i] - 'A'+ 10;
		else ttt = buf[i] - 'a'+ 10;
				tempui = (tempui<<4)+ttt;
	}

	return tempui;
}


//----------------------------------------------------------
//  [֐]: memset
//----------------------------------------------------------

void *memset(void *s, int c, unsigned int n)
{
	const unsigned char uc = c;
	unsigned char		*p = (unsigned char *)s;

	while (n-- > 0)
		*p++ = uc;

	return (s);
}




//-----------------------------------------------
/* USART1ŕ̑Ms֐ */
void USART_SendString(const char *str)
{
  const char *ptr = str;
  while (*ptr != '\0')
  {
    while(USART_GetFlagStatus(USART1, USART_FLAG_TXE) == RESET) {
    }
    USART_SendData(USART1, *ptr);
    ptr++;
  }
}


//-----------------------------------------------
/* USART1ňꕶ̑Ms֐ */
void USART_SendChar(char c)
{
  while(USART_GetFlagStatus(USART1, USART_FLAG_TXE) == RESET) {
  }
  USART_SendData(USART1, c);
}
