
//*************************************************************
// INCLUDE FILE                                                
//*************************************************************
#include <stdio.h>
#include <string.h>
#include <ctype.h>




//*************************************************************
// STATIC FUNCTION PROTOTYPE                                   
//*************************************************************
static void ETI_OutDev0( uint8_t uint8CharacterData );
static void ETI_OoutDev0( uint8_t uint8CharacterData );
static uint8_t ETI_InDev0( void );


void APE6_putc( unsigned char senddata );
uint8_t APE6_getchar( void );
int dscanf(const char *format, unsigned int *uipt);
int hex_dscanf(void);



uint8_t ETG_MonWrite( uint8_t* puint8Buffer, uint8_t uint8WriteNumber );
uint8_t ETG_MonGets( uint8_t* puint8Buffer );
uint8_t ETG_MonGetchar( void );



//***********************************************************************************************
// FUNCTION  : ETI_OutDev0
// OUTLINE   : string output to console. output CRLF(0x0A0D) at \n.                                                
//***********************************************************************************************
static void ETI_OutDev0( uint8_t uint8CharacterData )
{

	if( uint8CharacterData == '\n')
	{
		ETI_OoutDev0('\r');
		ETI_OoutDev0('\n');
	}
	else
	{
		ETI_OoutDev0( uint8CharacterData );
	}
}

//***********************************************************************************************
// FUNCTION  : ETI_OoutDev0
// OUTLINE   : transmission register write for console                                       
//***********************************************************************************************
static void ETI_OoutDev0( uint8_t uint8CharacterData )
{
    while(USART_GetFlagStatus(USART1, USART_FLAG_TXE) == RESET) {
    }
    USART_SendData(USART1, uint8CharacterData);

}

//***********************************************************************************************
// FUNCTION  : ETI_InDev0
// PARAMETER : void
// RETURN    : uint8_t
// OUTLINE   : receive 1Byte from console                                                    
// NOTE      : 2012/10/9                                                             
//***********************************************************************************************
static uint8_t ETI_InDev0( void )
{
	uint8_t uint8ReceiveFifoData;

	ETI_BufferCheckOff();
	// wait until getting string with DR=1
	while ((APE6_SCIFA0_SCASSR0 & 0x0002)==0x0);

	uint8ReceiveFifoData = APE6_SCIFA0_SCAFRDR0;
	APE6_SCIFA0_SCASSR0 &= ~0x0002;



	return( uint8ReceiveFifoData );
}

//***********************************************************************************************
// FUNCTION  : ETG_MonWrite
// PARAMETER : uint8_t* puint8Buffer, uint8_t uint8WriteNumber
// RETURN    : uint8_t  0:success     0xFF:fail
// OUTLINE   : transmission data write
// NOTE      : 2012/10/9
//***********************************************************************************************
uint8_t ETG_MonWrite( uint8_t* puint8Buffer, uint8_t uint8WriteNumber )
{
	uint8_t uint8Count;
	uint8_t uint8CharacterData;

	if ( uint8WriteNumber > 0 )
	{
		for ( uint8Count = uint8WriteNumber ;uint8Count > 0 ;uint8Count-- )
		{
			uint8CharacterData = *puint8Buffer;
			ETI_OutDev0( uint8CharacterData );
			puint8Buffer++;
		}
		return uint8Count;
	}
	else
	{
		return 0xFF;
	}
}

//***********************************************************************************************
// FUNCTION  : ETG_MonGetchar(void)
// RETURN    : uint8_t  success:character code    fail:0xFF
// OUTLINE   : get 1Byte
//***********************************************************************************************
uint8_t ETG_MonGetchar(void)
{
	uint8_t uint8CharacterData;

	if( ETI_MonRead( &uint8CharacterData, 1 ) == 1 ){
		return uint8CharacterData;
	}else{
		return (uint8_t)(0xFF);
	}
}


/*
 *============================================================================
 *  [֐]: APE6_getchar
 *  [@\]  : UART01
 *============================================================================
*/
uint8_t APE6_getchar( void )
{
	uint8_t uint8ReceiveFifoData;

	// wait until getting string with RDF=1
	while ((APE6_SCIFA0_SCASSR0 & 0x0002)==0x0);

	uint8ReceiveFifoData = APE6_SCIFA0_SCAFRDR0;
	APE6_SCIFA0_SCASSR0 &= ~0x0002;
	
	return( uint8ReceiveFifoData );

}


//-----------------------------------------------
  [֐]: APE6_putchar
  [@\]  : 1oCgo
//-----------------------------------------------
void APE6_putchar( unsigned char  c )
{
	APE6_putc( c );
}


//-----------------------------------------------
  [֐]: APE6_putc
  [@\]  : 1oCgo
//-----------------------------------------------
void APE6_putc( unsigned char senddata )
{
	if ( senddata == '\n') {

		// wait until SCFTDR2 is available
		// bit6 TEND Transmit End  0:transmission is in progress   1:transmission has been ended
		// bit5 TDFE Transmit FIFO Data Empty  0:transmit data exceeding trigger have written to SCAFTDR
		while(!(APE6_SCIFA0_SCASSR0 & 0x0060));

			APE6_SCIFA0_SCAFTDR0 = '\r';
	}

		/* Wait until TX FIFO is empty */
	while(!(APE6_SCIFA0_SCASSR0 & 0x0060));

	/* Send the character */
	// write transmission data to SCAFTDR4
	APE6_SCIFA0_SCAFTDR0 = senddata;


//-----------------------------------------------
  [֐]: dscanf
//-----------------------------------------------

int dscanf(const char *format, unsigned int *uipt)
{
#define DSCANF_STATUS_OK 1
#define DSCANF_STATUS_NG 0
#define DSCANF_STATUS_DEFAULT 0xfe
#define DSCANF_STATUS_CANCEL 0xff

	char c;
	char buf[10];
	int	pt,i,mode;
	unsigned int tempui;

	if((c = *format++) != '%' )	return(DSCANF_STATUS_NG);
	switch(c = *format++){
		case 'd':	mode = 'd'; break;
		case 'x':	mode = 'x'; break;
		default	: return(DSCANF_STATUS_NG);
	}

	pt=0;
	while ( (c = APE6_getchar())!= '\r' ){
		if(c==0x1b){	// ESC
			return(DSCANF_STATUS_CANCEL);
		}else if( (mode == 'd') &&  (c>='0' && c<='9') ){
			if(pt<10){
				buf[pt++] = c;
				APE6_putchar(c);
			}
		}else if((mode=='x') && ((c>='0' && c<='9') || (c>='A' && c<='F') ||(c>='a' && c<='f')) ){
			if(pt<8){
				buf[pt++] = c;
				APE6_putchar(c);
			}
		}else if(c==0x8||c==0x7f){	// BS or DEL
			if(pt>0){
				APE6_putchar(0x08);	// BS
				APE6_putchar(0x20);  // SPACE
				APE6_putchar(0x08);  // BS
				pt--;
			}
		}
	}

	if(pt==0) return(DSCANF_STATUS_NG);

	switch(mode){
		case 'd':{
			for(tempui=0,i=0;i<pt;i++){
				tempui = tempui*10 + buf[i] - '0';
				if(tempui>0x80000000) return(DSCANF_STATUS_NG);
			}
			break;
		}
		case 'x':{
			for(tempui=0,i=0;i<pt;i++){
				unsigned int ttt;
				if((buf[i]>='0') && (buf[i]<='9')) 	ttt = buf[i] - '0';
				else if ((buf[i]>='A') && (buf[i]<='F')) ttt = buf[i] - 'A'+ 10;
				else ttt = buf[i] - 'a'+ 10;

				tempui = (tempui<<4)+ttt;
			}
			break;
		}
		default:
				return(DSCANF_STATUS_NG);
	}

	*uipt = tempui;

	return(DSCANF_STATUS_OK);
}




//-----------------------------------------------
  [֐]: hex_dscanf
//-----------------------------------------------

int hex_dscanf(void)
{

	char c;
	char buf[10];
	int	pt,i;
	unsigned int tempui;

	pt=0;
	while ( (c = APE6_getchar())!= '\r' ){
		if((c>='0' && c<='9') || (c>='A' && c<='F') ||(c>='a' && c<='f')){
			if(pt<8){
				buf[pt++] = c;
				APE6_putchar(c);
			}
		}else if(c==0x8||c==0x7f){
			if(pt>0){
				APE6_putchar(0x08);
				APE6_putchar(0x20);
				APE6_putchar(0x08);
				pt--;
			}
		}
	}
	if(pt==0) return(0);

	for(tempui=0,i=0;i<pt;i++){
		int ttt;
		if((buf[i]>='0') && (buf[i]<='9')) ttt = buf[i] - '0';
		else if ((buf[i]>='A') && (buf[i]<='F')) ttt = buf[i] - 'A'+ 10;
		else ttt = buf[i] - 'a'+ 10;
				tempui = (tempui<<4)+ttt;
	}

	return tempui;
}



